// task 3 copies from 2 then iterates new features (all tasks will be done this way for purpose of demonstration and reference)

// include guard to prevent multiple incl;usions of header, alt pragma
#ifndef MY_STRING_HPP
#define MY_STRING_HPP

#include <iostream>
#include <cstring>

// my_string class setup, includes pointers and pointer refences
class my_string {

private:
    char* data;         
    int* ref_count;     

    void release() {
        if (ref_count) {
            (*ref_count)--;
            if (*ref_count == 0) {
                std::cout << "Release Memory: " << std::endl;
                delete[] data;
                delete ref_count;
            }
        }
    }

public:
    // constructors including c string and copy

    my_string() : data(nullptr), ref_count(new int(1)) {}

    my_string(const char* str) : data(new char[strlen(str) + 1]), ref_count(new int(1)) {
        std::strcpy(data, str);
    }

    my_string(const my_string& s) : data(s.data), ref_count(s.ref_count) {
        (*ref_count)++;
    }

    // Assignment operator include counter for references
    my_string& operator=(const my_string& s) {
        if (this != &s) {
            release();       
            data = s.data;
            ref_count = s.ref_count;
            (*ref_count)++;
        }
        return *this;
    }

    // Destructor
    ~my_string() {
        release();
    }

    // get uses spefic index
    char getChar(const int& i) const {
        if (i < 0 || !data || i >= std::strlen(data)) throw std::out_of_range("Index out of range");
        return data[i];
    }

    // set uses specific index
    void setChar(const int& i, const char& c) {
        if (i < 0 || !data || i >= std::strlen(data)) throw std::out_of_range("Index out of range");
        if (*ref_count > 1) {
            // If there are multiple owners, make a new copy (copy-on-write)
            char* new_data = new char[std::strlen(data) + 1];
            std::strcpy(new_data, data);
            release();  // Release old resources
            data = new_data;
            ref_count = new int(1);
        }
        data[i] = c;
    }

    // Print, changed to print with reference count
    void print() const {
        if (data) {
            std::cout << data << " [" << *ref_count << "]" << std::endl;
        } else {
            std::cout << "(null) [0]" << std::endl;
        }
    }
};

#endif
