
#ifndef BENCH_HPP
#define BENCH_HPP

#include <chrono>
#include <iostream>
#include <numeric>
#include <vector>
#include <algorithm>
#include <string>
#include <functional>


class bench{
private:
    // calculate and print usefull benchmark results
    static void Results(const std::string& testName, const std::vector<double>& durations){
        double sum = std::accumulate(durations.begin(), durations.end(), 0.0);
        double mean = sum/durations.size();

        std::vector<double> sortedDurations = durations;
        std::sort(sortedDurations.begin(), sortedDurations.end());
        double median = sortedDurations[sortedDurations.size() / 2];

        std::cout << "Benchmark: " << testName << std::endl;
        std::cout << "Mean: " << mean << std::endl;
        std::cout << "Median: " << median << std::endl;
        std::cout << "Runs: " << durations.size() << std::endl;
        std::cout << "------------------------------" << std::endl;
    }

public:
    // template function for benchmark tests taking arguments so usefull for multiple tests
    template <typename Func, typename... Args>
    static void Run(const std::string& testName, Func func, Args... args){
        using Clock = std::chrono::system_clock;
        std::vector<double> durations;

        // basic iterative for loop
        for (int i = 0; i < 10; ++i){
            auto start = Clock::now();
            func(args...);
            auto end = Clock::now();
            std::chrono::duration<double, std::milli> duration = end-start;
            durations.push_back(duration.count());
        }
        Results(testName, durations);
    }
};


#endif