#include "bench.hpp"
#include "downBump.hpp"
#include <vector>
#include <cstdlib>


// Benchmark allocating many small blocks of memory
void smallBlockBench(){
    downBump allocator(1024);
    for (int i = 0; i < 100; ++i){
        allocator.alloc<int>();
    }
}

// Benchmark allocating many large blocks of memory
void largeBlockBench(){
    downBump allocator(1024*1024);
    for (int i = 0; i < 100; ++i){
        allocator.alloc<double>(); 
    }
}

// Benchmark allocating mixed sized and mixed type blocks
void mixedBlockBench(){
    downBump allocator(1024*1024);
    for (int i = 0; i < 50; i++){
        allocator.alloc<int>(1);
        allocator.alloc<double>(16);
        allocator.alloc<char>(256);
    }
}

// Benchmark simulates allocation to reallocation cycle
void allocateReset(){
    downBump allocator(1024*1024);
    for (int i = 0; i<50; ++i){
        allocator.alloc<int>(64);
    }
    allocator.reset();
}

// Benchmark stress tests by allocting whole heap in small chunks.
void stressTest(){
    downBump allocator(1024);
    while(true){
        try{
            allocator.alloc<char>(16);
        }
        catch (const std::bad_alloc&){
            break;
        }
    }
}


// runs any benchmark functions called
int main(){
    bench::Run("Small block benchmark", smallBlockBench);
    bench::Run("Large block benchmark", largeBlockBench);
    bench::Run("Mixed block benchmark", mixedBlockBench);
    bench::Run("Allocation and reset benchmark", allocateReset);
    bench::Run("Stress test benchmark", stressTest);

    return 0;
}